@@ -295,6 +295,21 @@ void v4lconvert_yuyv_to_yuv420(const unsigned char *src, unsigned char *dest,
 	}
 }
 
+void v4lconvert_nv16_to_yuyv(const unsigned char *src, unsigned char *dest,
+		int width, int height)
+{
+	const unsigned char *y, *cbcr;
+	int count = 0;
+
+	y = src;
+	cbcr = src + width*height;
+
+	while (count++ < width*height) {
+		*dest++ = *y++;
+		*dest++ = *cbcr++;
+	}
+}
+
 void v4lconvert_yvyu_to_bgr24(const unsigned char *src, unsigned char *dest,
 		int width, int height, int stride)
 {
@@ -755,3 +770,78 @@ void v4lconvert_rgb32_to_rgb24(const unsigned char *src, unsigned char *dest,
 		}
 	}
 }
+
+static void hsvtorgb(const unsigned char *hsv, unsigned char *rgb,
+		     unsigned char hsv_enc)
+{
+	/* From http://stackoverflow.com/questions/3018313/ */
+	uint8_t region;
+	uint8_t remain;
+	uint8_t p, q, t;
+
+	if (!hsv[1]) {
+		rgb[0] = rgb[1] = rgb[2] = hsv[2];
+		return;
+	}
+
+	if (hsv_enc == V4L2_HSV_ENC_256) {
+		region = hsv[0] / 43;
+		remain = (hsv[0] - (region * 43)) * 6;
+
+	} else {
+		int aux;
+
+		region = hsv[0] / (180/6);
+
+		/* Remain must be scaled to 0..255 */
+		aux = (hsv[0] % (180/6)) * 6 * 256;
+		aux /= 180;
+		remain = aux;
+	}
+
+	p = (hsv[2] * (255 - hsv[1])) >> 8;
+	q = (hsv[2] * (255 - ((hsv[1] * remain) >> 8))) >> 8;
+	t = (hsv[2] * (255 - ((hsv[1] * (255 - remain)) >> 8))) >> 8;
+
+	switch (region)	{
+	case 0:
+		rgb[0] = hsv[2]; rgb[1] = t; rgb[2] = p;
+		break;
+	case 1:
+		rgb[0] = q; rgb[1] = hsv[2]; rgb[2] = p;
+		break;
+	case 2:
+		rgb[0] = p; rgb[1] = hsv[2]; rgb[2] = t;
+		break;
+	case 3:
+		rgb[0] = p; rgb[1] = q; rgb[2] = hsv[2];
+		break;
+	case 4:
+		rgb[0] = t; rgb[1] = p; rgb[2] = hsv[2];
+		break;
+	default:
+		rgb[0] = hsv[2]; rgb[1] = p; rgb[2] = q;
+		break;
+	}
+
+}
+
+void v4lconvert_hsv_to_rgb24(const unsigned char *src, unsigned char *dest,
+		int width, int height, int bgr, int Xin, unsigned char hsv_enc){
+	int j, k;
+	int bppIN = Xin / 8;
+	unsigned char rgb[3];
+
+	src += bppIN - 3;
+
+	while (--height >= 0)
+		for (j = 0; j < width; j++) {
+			hsvtorgb(src, rgb, hsv_enc);
+			for (k = 0; k < 3; k++)
+				if (bgr && k < 3)
+					*dest++ = rgb[2-k];
+				else
+					*dest++ = rgb[k];
+			src += bppIN;
+		}
+}
