@@ -389,6 +389,11 @@ static inline bool v4l_has_sdr_out(const struct v4l_fd *f)
 	return v4l_g_caps(f) & V4L2_CAP_SDR_OUTPUT;
 }
 
+static inline bool v4l_has_meta_cap(const struct v4l_fd *f)
+{
+	return v4l_g_caps(f) & V4L2_CAP_META_CAPTURE;
+}
+
 static inline bool v4l_has_touch(const struct v4l_fd *f)
 {
 	return v4l_g_caps(f) & V4L2_CAP_TOUCH;
@@ -435,6 +440,8 @@ static inline __u32 v4l_determine_type(const struct v4l_fd *f)
 		return V4L2_BUF_TYPE_SDR_CAPTURE;
 	if (v4l_has_sdr_out(f))
 		return V4L2_BUF_TYPE_SDR_OUTPUT;
+	if (v4l_has_meta_cap(f))
+		return V4L2_BUF_TYPE_META_CAPTURE;
 	return 0;
 }
 
@@ -582,6 +589,9 @@ static inline void v4l_format_s_pixelformat(struct v4l2_format *fmt, __u32 pixel
 	case V4L2_BUF_TYPE_VBI_OUTPUT:
 		fmt->fmt.vbi.sample_format = pixelformat;
 		break;
+	case V4L2_BUF_TYPE_META_CAPTURE:
+		fmt->fmt.meta.dataformat = pixelformat;
+		break;
 	}
 }
 
@@ -600,6 +610,8 @@ static inline __u32 v4l_format_g_pixelformat(const struct v4l2_format *fmt)
 	case V4L2_BUF_TYPE_VBI_CAPTURE:
 	case V4L2_BUF_TYPE_VBI_OUTPUT:
 		return fmt->fmt.vbi.sample_format;
+	case V4L2_BUF_TYPE_META_CAPTURE:
+		return fmt->fmt.meta.dataformat;
 	default:
 		return 0;
 	}
@@ -928,6 +940,8 @@ v4l_format_g_sizeimage(const struct v4l2_format *fmt, unsigned plane)
 	case V4L2_BUF_TYPE_SDR_CAPTURE:
 	case V4L2_BUF_TYPE_SDR_OUTPUT:
 		return plane ? 0 : fmt->fmt.sdr.buffersize;
+	case V4L2_BUF_TYPE_META_CAPTURE:
+		return plane ? 0 : fmt->fmt.meta.buffersize;
 	default:
 		return 0;
 	}
@@ -1046,6 +1060,11 @@ static inline bool v4l_type_is_sdr(unsigned type)
 	      type == V4L2_BUF_TYPE_SDR_OUTPUT;
 }
 
+static inline bool v4l_type_is_meta(unsigned type)
+{
+       return type == V4L2_BUF_TYPE_META_CAPTURE;
+}
+
 static inline unsigned v4l_type_invert(unsigned type)
 {
 	if (v4l_type_is_planar(type))
