@@ -332,6 +332,7 @@ struct parse_event key_events[] = {
 	{"BTN_TOOL_MOUSE", 0x146},
 	{"BTN_TOOL_LENS", 0x147},
 	{"BTN_TOOL_QUINTTAP", 0x148},
+	{"BTN_STYLUS3", 0x149},
 	{"BTN_TOUCH", 0x14a},
 	{"BTN_STYLUS", 0x14b},
 	{"BTN_STYLUS2", 0x14c},
@@ -510,6 +511,7 @@ struct parse_event key_events[] = {
 	{"KEY_APPSELECT", 0x244},
 	{"KEY_SCREENSAVER", 0x245},
 	{"KEY_VOICECOMMAND", 0x246},
+	{"KEY_ASSISTANT", 0x247},
 	{"KEY_BRIGHTNESS_MIN", 0x250},
 	{"KEY_BRIGHTNESS_MAX", 0x251},
 	{"KEY_KBDINPUTASSIST_PREV", 0x260},
@@ -535,7 +537,8 @@ struct parse_event key_events[] = {
 	{"KEY_UNMUTE", 0x274},
 	{"KEY_FASTREVERSE", 0x275},
 	{"KEY_SLOWREVERSE", 0x276},
-	{"KEY_DATA", 0x275},
+	{"KEY_DATA", 0x277},
+	{"KEY_ONSCREEN_KEYBOARD", 0x278},
 	{"BTN_TRIGGER_HAPPY", 0x2c0},
 	{"BTN_TRIGGER_HAPPY1", 0x2c0},
 	{"BTN_TRIGGER_HAPPY2", 0x2c1},
