@@ -53,7 +53,9 @@ static const flag_def in_status_def[] = {
 	{ V4L2_IN_ST_NO_COLOR,    "no color" },
 	{ V4L2_IN_ST_HFLIP,       "hflip" },
 	{ V4L2_IN_ST_VFLIP,       "vflip" },
-	{ V4L2_IN_ST_NO_H_LOCK,   "no hsync lock." },
+	{ V4L2_IN_ST_NO_H_LOCK,   "no hsync lock" },
+	{ V4L2_IN_ST_NO_V_LOCK,   "no vsync lock" },
+	{ V4L2_IN_ST_NO_STD_LOCK, "no standard format lock" },
 	{ V4L2_IN_ST_COLOR_KILL,  "color kill" },
 	{ V4L2_IN_ST_NO_SYNC,     "no sync lock" },
 	{ V4L2_IN_ST_NO_EQU,      "no equalizer lock" },
@@ -69,6 +71,34 @@ static std::string status2s(__u32 status)
 	return status ? flags2s(status, in_status_def) : "ok";
 }
 
+static const char *inputtype2s(__u32 type)
+{
+	switch (type) {
+	case V4L2_INPUT_TYPE_TUNER:
+		return "Tuner";
+	case V4L2_INPUT_TYPE_CAMERA:
+		return "Camera";
+	case V4L2_INPUT_TYPE_TOUCH:
+		return "Touch";
+	default:
+		return "Unknown";
+	}
+}
+
+static const char *outputtype2s(__u32 type)
+{
+	switch (type) {
+	case V4L2_OUTPUT_TYPE_MODULATOR:
+		return "Modulator";
+	case V4L2_OUTPUT_TYPE_ANALOG:
+		return "Analog";
+	case V4L2_OUTPUT_TYPE_ANALOGVGAOVERLAY:
+		return "Analog VGA Overlay";
+	default:
+		return "Unknown";
+	}
+}
+
 
 static const flag_def input_cap_def[] = {
 	{ V4L2_IN_CAP_DV_TIMINGS, "DV timings" },
@@ -121,7 +151,9 @@ void io_set(int fd)
 			printf("Video input set to %d", input);
 			vin.index = input;
 			if (test_ioctl(fd, VIDIOC_ENUMINPUT, &vin) >= 0)
-				printf(" (%s: %s)", vin.name, status2s(vin.status).c_str());
+				printf(" (%s: %s, %s)", vin.name,
+				       inputtype2s(vin.type),
+				       status2s(vin.status).c_str());
 			printf("\n");
 		}
 	}
@@ -192,7 +224,7 @@ void io_list(int fd)
 				printf("\n");
 			printf("\tInput       : %d\n", vin.index);
 			printf("\tName        : %s\n", vin.name);
-			printf("\tType        : 0x%08X\n", vin.type);
+			printf("\tType        : 0x%08X (%s)\n", vin.type, inputtype2s(vin.type));
 			printf("\tAudioset    : 0x%08X\n", vin.audioset);
 			printf("\tTuner       : 0x%08X\n", vin.tuner);
 			printf("\tStandard    : 0x%016llX (%s)\n", (unsigned long long)vin.std,
@@ -213,7 +245,7 @@ void io_list(int fd)
 				printf("\n");
 			printf("\tOutput      : %d\n", vout.index);
 			printf("\tName        : %s\n", vout.name);
-			printf("\tType        : 0x%08X\n", vout.type);
+			printf("\tType        : 0x%08X (%s)\n", vout.type, outputtype2s(vout.type));
 			printf("\tAudioset    : 0x%08X\n", vout.audioset);
 			printf("\tStandard    : 0x%016llX (%s)\n", (unsigned long long)vout.std,
 					std2s(vout.std).c_str());
